/**
 * Copyright (c) 2011-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.commons.log;

import java.util.HashMap;
import java.util.Map;

import org.junit.After;
import org.junit.Assert;
import org.junit.Test;
import org.slf4j.MDC;

/**
 * @author Mathieu Carrolle - EBM WebSourcing
 */
public class PetalsExecutionContextTest extends Assert {

    @After
    public void after() {
        PetalsExecutionContext.clearAll();
    }

    @Test
    public void testPutFlowInstanceId() throws Exception {
        final String flowInstanceId = "flowInstanceId";
        PetalsExecutionContext.putFlowInstanceId(flowInstanceId);
        Map<String, String> expectedProperties = new HashMap<>();
        expectedProperties.put(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME, flowInstanceId);
        assertEquals(expectedProperties, MDC.getCopyOfContextMap());
    }

    @Test
    public void testInitFlowInstanceId() throws Exception {
        final FlowAttributes fa = PetalsExecutionContext.initFlowAttributes();

        final Map<String, String> contextualProperties = MDC.getCopyOfContextMap();

        String flowInstanceId = contextualProperties.get(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME);
        assertNotNull(flowInstanceId);
        assertTrue(!flowInstanceId.isEmpty());
        assertEquals(flowInstanceId, fa.getFlowInstanceId());

        String flowStepId = contextualProperties.get(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME);
        assertNotNull(flowStepId);
        assertTrue(!flowStepId.isEmpty());
        assertEquals(flowStepId, fa.getFlowStepId());
    }
    
    @Test
    public void testPutFlowStepId() throws Exception {
        final String flowStepId = "flowStepId";
        PetalsExecutionContext.putFlowStepId(flowStepId);
        Map<String, String> expectedProperties = new HashMap<>();
        expectedProperties.put(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME, flowStepId);
        assertEquals(expectedProperties, MDC.getCopyOfContextMap());
    }

    @Test
    public void testNextStepIdNoInstance() throws Exception {
        PetalsExecutionContext.nextFlowStepId();
        String flowStepId = MDC.get(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME);
        assertNull(flowStepId);
    }

    @Test
    public void testNextFlowStepId() {
        final FlowAttributes initFa = PetalsExecutionContext.initFlowAttributes();
        final FlowAttributes fa = PetalsExecutionContext.nextFlowStepId();
        assertEquals(initFa.getFlowInstanceId(), MDC.get(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME));
        assertNotEquals(initFa.getFlowStepId(), MDC.get(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME));
        assertEquals(MDC.get(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME), fa.getFlowStepId());
        assertEquals(MDC.get(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME), fa.getFlowInstanceId());
    }

    @Test
    public void testGetFlowAttributesButNoneInContext() throws Exception {
        assertNull(PetalsExecutionContext.getFlowAttributes());
    }

    @Test
    public void testGetFlowAttributesButFlowInstanceIdMissingInContext() throws Exception {
        MDC.put(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME, "flowStepId1");
        assertNull(PetalsExecutionContext.getFlowAttributes());
    }

    @Test
    public void testGetFlowAttributesButFlowStepIdMissingInContext() throws Exception {
        MDC.put(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME, "flowInstanceId1");
        assertNull(PetalsExecutionContext.getFlowAttributes());
    }

    @Test
    public void testGetFlowAttributes() throws Exception {
        final String flowInstanceId = "flowInstanceId1";
        final String flowStepId = "flowStepId1";
        MDC.put(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME, flowInstanceId);
        MDC.put(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME, flowStepId);
        FlowAttributes expectedFlowAttributes = new FlowAttributes(flowInstanceId, flowStepId);
        assertEquals(expectedFlowAttributes, PetalsExecutionContext.getFlowAttributes());
    }

    @Test
    public void testPutFlowAttributes() throws Exception {
        final String flowInstanceId = "flowInstanceId1";
        final String flowStepId = "flowStepId1";
        FlowAttributes flowAttributes = new FlowAttributes(flowInstanceId, flowStepId);

        PetalsExecutionContext.putFlowAttributes(flowAttributes);
        assertEquals(flowInstanceId, MDC.get(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME));
        assertEquals(flowStepId, MDC.get(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME));

    }
}
